<?php
namespace Magenest\Salesforce\Controller\Adminhtml\Mass;

use Magenest\Salesforce\Helper\Constant;
use Magenest\Salesforce\Model\ResourceModel\Queue as QueueResource;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Ui\Component\MassAction\Filter;
use Psr\Log\LoggerInterface;

/**
 * Class AddToQueue
 * @package Magenest\Salesforce\Controller\Adminhtml\Mass
 */
class AddToQueue extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Magenest_Salesforce::queue';

    /** @var QueueResource  */
    protected $_queueResource;

    /**
     * @var string
     */
    protected $redirectUrl = '*/*/index';

    /** @var Filter  */
    protected $_filter;

    /** @var LoggerInterface  */
    protected $_logger;

    /**
     * @var array
     */
    protected $_salesforceObject = [
        'customer' => [
            Constant::SALESFORCE_LEAD_OBJECT,
            Constant::SALESFORCE_CONTACT_OBJECT,
            Constant::SALESFORCE_ACCOUNT_OBJECT
        ],
        'order' => [
            Constant::SALESFORCE_ORDER_OBJECT,
            Constant::SALESFORCE_OPPORTUNITY_OBJECT
        ],
        'product' => [
            Constant::SALESFORCE_PRODUCT2_OBJECT
        ],
        'campaign' => [
            Constant::SALESFORCE_CAMPAIGN_OBJECT
        ]
    ];

    /**
     * AddToQueue constructor.
     * @param QueueResource $queueResource
     * @param Filter $filter
     * @param LoggerInterface $logger
     * @param Context $context
     */
    public function __construct(
        QueueResource $queueResource,
        Filter $filter,
        LoggerInterface $logger,
        Context $context
    ) {
        $this->_queueResource = $queueResource;
        $this->_filter = $filter;
        $this->_logger = $logger;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Backend\Model\View\Result\Redirect|\Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        try {
            $params = $this->getRequest()->getParams();
            $selectedParam = $this->getRequest()->getParam(Filter::SELECTED_PARAM);
            $type = $params['type'] ?? null;
            if (!isset($params['type'])) {
                throw new \Exception(__("Object is not determined"));
            }
            if (isset($this->_salesforceObject[$type])) {
                $salesforceObject = $this->_salesforceObject[$type];
                foreach ($salesforceObject as $object) {
                    $this->_queueResource->publishData($selectedParam, $object);
                }
                $this->messageManager->addSuccessMessage(__('A total of %1 data have been added to queue.', count($selectedParam)));
                if ($type == 'customer') {
                    return $resultRedirect->setPath('customer/index/index');
                } elseif ($type == 'order') {
                    return $resultRedirect->setPath('sales/order/index');
                } elseif ($type == 'product') {
                    return $resultRedirect->setPath('catalog/product/index');
                } else {
                    return $resultRedirect->setPath($this->redirectUrl);
                }

            }
        } catch (\Exception $exception) {
            $this->_logger->debug($exception->getMessage());
            $this->messageManager->addErrorMessage($exception->getMessage());
            /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
            $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
            return $resultRedirect->setPath($this->redirectUrl);
        }
    }
}
