<?php
namespace Magenest\Salesforce\Controller\Adminhtml\Sync;

use Magenest\Salesforce\Model\Sync;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class Order
 * @package Magenest\Salesforce\Controller\Adminhtml\Sync
 */
class Order extends Action
{
    /**
     * @var Sync\Order
     */
    protected $_order;

    /**
     * Customer constructor.
     *
     * @param Context $context
     * @param Sync\Order $order
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     */
    public function __construct(
        Context $context,
        Sync\Order $order
    ) {
        $this->_order = $order;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|\Magento\Framework\View\Result\Layout
     */
    public function execute()
    {
        try {
            $orderIncrementId = $this->getRequest()->getParam('increment_id');
            $orderId = $this->getRequest()->getParam('order_id');
            if ($orderIncrementId) {
                $this->_order->sync($orderIncrementId);
                $this->messageManager->addSuccessMessage(
                    __('Order sync process complete, check out Reports for result.')
                );
            } else {
                $this->messageManager->addNoticeMessage(
                    __('No order has been selected')
                );
            }
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage(
                __('Something happen during syncing process. Detail: ' . $e->getMessage())
            );
        }
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        if (isset($orderId)) {
            $resultRedirect->setPath('sales/order/view', ['order_id' => $orderId]);
        } else {
            $resultRedirect->setUrl($this->_redirect->getRefererUrl());
        }
        return $resultRedirect;
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magenest_Salesforce::config_salesforce');
    }
}
