<?php
/**
 * Copyright © 2015 Magenest. All rights reserved.
 * See COPYING.txt for license details.
 *
 * Magenest_Salesforce extension
 * NOTICE OF LICENSE
 *
 * @category Magenest
 * @package  Magenest_Salesforce
 * @author   ThaoPV
 */

namespace Magenest\Salesforce\Observer\Order;

use Magenest\Salesforce\Model\Config\Source\SyncMode;
use Magenest\Salesforce\Model\Queue;
use Magenest\Salesforce\Observer\SyncObserver;
use Magento\Framework\Event\Observer;

/**
 * Class Create
 * @package Magenest\Salesforce\Observer\Order
 */
class Create extends SyncObserver
{
    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        try {
            $order = $observer->getEvent()->getOrder();
            if (!$order instanceof \Magento\Sales\Model\Order) {
                $orderIds = $observer->getEvent()->getOrderIds();
                if (is_array($orderIds) && !empty($orderIds)) {
                    $orderId = reset($orderIds);
                    $order = $this->orderRepository->get($orderId);
                }
            }
            if ($order instanceof \Magento\Sales\Model\Order || $order instanceof \Magento\Sales\Api\Data\OrderInterface) {
                $orderIncrementId = $order->getIncrementId();
                if ($this->getEnabledSyncConfig('order')) {
                    if ($this->getSyncConfigMode('order') == SyncMode::ADD_TO_QUEUE) {
                        $this->addToQueue(Queue::TYPE_ORDER, $orderIncrementId);
                    } else {
                        $this->_order->sync($orderIncrementId);
                    }
                }
                if ($this->getEnabledSyncConfig('opportunity')) {
                    if ($this->getSyncConfigMode('opportunity') == SyncMode::ADD_TO_QUEUE) {
                        $this->addToQueue(Queue::TYPE_OPPORTUNITY, $orderIncrementId);
                    } else {
                        $this->_opportunity->sync($orderIncrementId);
                    }
                }
            } else {
                throw new \Exception(__('Order not exist.'));
            }
        } catch (\Exception $e) {
            $this->logger->info($e->getMessage());
        }
    }
}
